//
//  MyFirstMapAppViewController.m
//  MyFirstMapApp
//
//  Created by James Richards on 5/4/10.
//  Copyright Artisan Global LLC 2010. All rights reserved.
//

#import "MyFirstMapAppViewController.h"

@implementation MyFirstMapAppViewController

@synthesize mapView;
@synthesize streetView;
@synthesize imageryView;
@synthesize reliefView;
@synthesize locationManager;
@synthesize locateButton;

- (void)viewWillAppear:(BOOL)animated {
	// Setup location manager
	NSLog(@"Starting core location");
	self.locationManager = [[CLLocationManager alloc] init];
	self.locationManager.desiredAccuracy = kCLLocationAccuracyNearestTenMeters;
	self.locationManager.delegate = self;
	[self.locationManager startUpdatingLocation];
	self.locateButton.enabled = NO;
}

// Implement viewDidLoad to do additional setup after loading the view, typically from a nib.
- (void)viewDidLoad {
    [super viewDidLoad];
	
	self.mapView.mapViewDelegate = self;
	
	AGSTiledMapServiceLayer *streetLayer = [[AGSTiledMapServiceLayer alloc] 
											initWithURL:[NSURL URLWithString:kTiledStreetMapServiceURL]];
	self.streetView = [self.mapView addMapLayer:streetLayer withName:@"Street"];
	[streetLayer release];
	
	AGSTiledMapServiceLayer *imageryLayer = [[AGSTiledMapServiceLayer alloc] 
											 initWithURL:[NSURL URLWithString:kTiledImageryMapServiceURL]];
	self.imageryView = [self.mapView addMapLayer:imageryLayer withName:@"Imagery"];
	[imageryLayer release];
	
	AGSTiledMapServiceLayer *reliefLayer = [[AGSTiledMapServiceLayer alloc] 
											initWithURL:[NSURL URLWithString:kTiledReliefMapServiceURL]];
	self.reliefView = [self.mapView addMapLayer:reliefLayer withName:@"Relief"];
	[reliefLayer release];

	self.streetView.hidden = NO;
	self.imageryView.hidden = YES;
	self.reliefView.hidden = YES;

	// Create a graphics layer to display the push pin
	AGSGraphicsLayer *graphicsLayer = [AGSGraphicsLayer graphicsLayer];
	[self.mapView addMapLayer:graphicsLayer withName:@"GraphicsLayer"];
}

// Override to allow orientations other than the default portrait orientation.
- (BOOL)shouldAutorotateToInterfaceOrientation:(UIInterfaceOrientation)interfaceOrientation {
    // Return YES for supported orientations
    return YES;
}

- (void)didReceiveMemoryWarning {
	// Releases the view if it doesn't have a superview.
    [super didReceiveMemoryWarning];
	
	// Release any cached data, images, etc that aren't in use.
}

- (void) viewWillDisappear:(BOOL)animated {
	[super viewWillDisappear:animated];
	NSLog(@"Shutting down core location");
	[self.locationManager stopUpdatingLocation];
	self.locationManager = nil;
}

- (void)viewDidUnload {
	// Release any retained subviews of the main view.
	// e.g. self.myOutlet = nil;
}

- (void)dealloc {
	self.mapView = nil;
	self.streetView = nil;
	self.imageryView = nil;
	self.reliefView = nil;
	
	[locateButton release];
    [super dealloc];
}

- (IBAction)toggleLayer:(id)sender {
	
	self.streetView.hidden = (((UISegmentedControl *)sender).selectedSegmentIndex != 0);
	self.imageryView.hidden = (((UISegmentedControl *)sender).selectedSegmentIndex != 1);
	self.reliefView.hidden = (((UISegmentedControl *)sender).selectedSegmentIndex != 2);	
}

- (IBAction)showLocation:(id)sender {
	NSLog(@"Show Location");
	
	CLLocation *location = self.locationManager.location;
	double lat = location.coordinate.latitude;
	double lon = location.coordinate.longitude;
	NSLog(@"%.3f, %.3f", lat, lon);
	
	double size = 0.05;
	AGSEnvelope *envelope = [AGSEnvelope envelopeWithXmin:lon - size 
													 ymin:lat - size
													 xmax:lon + size 
													 ymax:lat + size
										 spatialReference:self.mapView.spatialReference];
	[self.mapView zoomToEnvelope:envelope animated:YES];
	
	// Get reference to the graphics layer
	id<AGSLayerView> graphicsLayerView = [self.mapView.mapLayerViews objectForKey:@"GraphicsLayer"];
	AGSGraphicsLayer *graphicsLayer = (AGSGraphicsLayer*)graphicsLayerView.agsLayer;
	
	// Clear graphics
	[graphicsLayer removeAllGraphics];
	
	// Create a marker symbol using the Location.png graphic
	AGSPictureMarkerSymbol *markerSymbol = [AGSPictureMarkerSymbol pictureMarkerSymbolWithImageNamed:@"Location.png"];
	
	// Create a new graphic using the location and marker symbol
	AGSGraphic* graphic = [AGSGraphic graphicWithGeometry:[envelope center]
												   symbol:markerSymbol
											   attributes:nil
											 infoTemplate:nil];
	
	// Add the graphic to the graphics layer
	[graphicsLayer addGraphic:graphic];
}


- (void) locationManager:(CLLocationManager *)manager didUpdateToLocation:(CLLocation *)newLocation fromLocation:(CLLocation *)oldLocation {
	NSLog(@"Core location has a new position");
	self.locateButton.enabled = YES;
}

- (void) locationManager:(CLLocationManager *)manager didFailWithError:(NSError *)error {
	NSLog(@"Core location failed to get position");
	self.locateButton.enabled = NO;
}

#pragma mark AGSMapViewDelegate

//called when the map view is loaded (after the view is loaded) 
- (void)mapViewDidLoad:(AGSMapView *)mapView {
	
	//create extent to be used as default
	AGSEnvelope *envelope = [AGSEnvelope envelopeWithXmin:-124.83145667
                                                     ymin:30.49849464
                                                     xmax:-113.91375495
                                                     ymax:44.69150688
                                         spatialReference:mapView.spatialReference];
    
	//call method to set extent, pass in envelope
	[self.mapView performSelector:@selector(zoomToEnvelope:animated:) 
                       withObject:envelope
                       afterDelay:0.5];
	
}	

@end